@@ -1,12 +1,12 @@
+#include <Python.h>
+#include <structmember.h>
+#include <numpy/arrayobject.h>
+
 #include <cstdlib>
 #include <cstdlib>
 #include <iostream>
 #include <vector>
 
-#include <Python.h>
-#include <structmember.h>
-#include <numpy/arrayobject.h>
-
 #include "array_col_reader.hpp"
 #include "_line_to_array.hpp"
 
@@ -98,7 +98,7 @@ array_col_reader_parse_missing_vals(PyObject * iterator, long num_types, bool &
         parsed_missing_vals[j] = make_pair(missing_vals[j], missing_vals[j] + strlen(missing_vals[j]));
         if (strlen(missing_vals[j]) >= max_field_len - 1) {
             err = true;
-            PyErr_Format(PyExc_IndexError, "Missing value too long - max %ld", max_field_len - 1);
+            PyErr_Format(PyExc_IndexError, "Missing value too long - max %d", max_field_len - 1);
             return NULL;
         }
     }
@@ -154,7 +154,7 @@ parse_arrays(PyObject * iterator, long & num, bool & err)
 }
 
 extern "C" PyObject *
-array_col_reader(PyObject *module, PyObject *args, PyObject *keyword_args)
+array_col_reader_new(PyTypeObject * type, PyObject *args, PyObject *keyword_args)
 {
     ArrayColReader * const self = PyObject_GC_New(ArrayColReader, &ArrayColReaderType);
     if (self == NULL) {
@@ -189,7 +189,6 @@ array_col_reader(PyObject *module, PyObject *args, PyObject *keyword_args)
         return NULL;
     }
 
-    // TRACE("Here");   
     bool err = false;
     self->input_iter = PyObject_GetIter(iterator);
     self->cols = parse_longs(cols_iterator, self->num_cols, err);
@@ -197,15 +196,12 @@ array_col_reader(PyObject *module, PyObject *args, PyObject *keyword_args)
     self->copy_cols = parse_longs(copy_cols_iterator, self->num_copy_cols, err);
     self->types = parse_longs(types_iterator, self->num_types, err);
     self->missing_vals = array_col_reader_parse_missing_vals(missing_vals_iterator, self->num_types, err);
-    // TRACE("Here");   
     self->bufs = parse_arrays(bufs_iterator, self->num_types, err);
-    // TRACE("Here");   
     if (self->input_iter == NULL || err) {
         PyErr_SetString(PyExc_TypeError, "Failed to parse stuff");
         Py_DECREF(self);
         return NULL;
     }
-    // TRACE("Parsed");   
 
     Py_INCREF(self->input_iter);
 
@@ -220,7 +216,6 @@ array_col_reader_copy_int_parsed_col(
     PyArrayObject * ret, 
     long i)
 {
-    // TRACE(i << " " << string(t.first, t.second) << " p " << (char *)(ret->data));
     bool err;
     ((long *)ret->data)[i] = parsed_to_long(t.first == t.second? missing_val : t, err);
     return !err;
@@ -233,7 +228,6 @@ array_col_reader_copy_float_parsed_col(
     PyArrayObject * ret, 
     long i)
 {
-    // TRACE(i << " " << string(t.first, t.second));
     bool err;
     ((double *)ret->data)[i] = parsed_to_double(t.first == t.second? missing_val : t, err);
     return !err;
@@ -247,7 +241,6 @@ array_col_reader_copy_str_parsed_col(
     long i)
 {
     char buf[4];
-    // TRACE(i << " " << string(t.first, t.second));
     const parsed_t & pt = t.first == t.second? missing_val : t;
     sprintf(buf, "%03d", static_cast<int>(distance(pt.first, pt.second)));
     copy(buf, buf + 3, (char *)ret->data + i * max_field_len);
@@ -341,16 +334,12 @@ array_col_reader_parse_line(ArrayColReader * self, parsed_t parsed[max_num_cols]
 extern "C" PyObject *
 array_col_reader_iternext(ArrayColReader * self)
 {
-    // TRACE("Starting iter chunk");
     long i;
     parsed_t parsed[max_num_cols];
     for (i = 0; i  < self->max_elems; ++i) {
-        // TRACE("Iter chunk row " << i);
     
-        if (!array_col_reader_parse_line(self, parsed)) {
-            // TRACE("Returning " << i);
+        if (!array_col_reader_parse_line(self, parsed)) 
             break;
-        }
         
         if (self->copy_cols != NULL?
                 !array_col_reader_copy_parsed_from_inds(self, parsed, i) :
@@ -358,7 +347,6 @@ array_col_reader_iternext(ArrayColReader * self)
             return NULL;
     }
 
-    // TRACE("Returning " << i);
     return pyint_from_long(i);
 }
 
@@ -366,6 +354,8 @@ extern "C" int
 array_col_reader_clear(ArrayColReader * self)
 {
     Py_CLEAR(self->input_iter);
+    for (long j = 0; j < self->num_types; ++j)
+        Py_CLEAR(self->bufs[j]);
     return 0;
 }
 
@@ -381,16 +371,16 @@ PyDoc_STRVAR(ArrayColReaderType_doc, "");
 
 PyTypeObject ArrayColReaderType = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "dagpype_c.array_col_reader",               /*tp_name*/
+    "dagpype_c.ArrayColReader",               /*tp_name*/
     sizeof(ArrayColReader),                   /*tp_basicsize*/
     0,                                      /*tp_itemsize*/
     /* methods */
     (destructor)array_col_reader_dealloc,          /*tp_dealloc*/
-    0,                           /*tp_print*/
-    0,                         /*tp_getattr*/
-    0,                         /*tp_setattr*/
-    0,                             /*tp_compare*/
-    0,                            /*tp_repr*/
+    0,                                         /*tp_print*/
+    0,                                         /*tp_getattr*/
+    0,                                         /*tp_setattr*/
+    0,                                          /*tp_compare*/
+    0,                                        /*tp_repr*/
     0,                                      /*tp_as_number*/
     0,                                      /*tp_as_sequence*/
     0,                                      /*tp_as_mapping*/
@@ -411,5 +401,13 @@ PyTypeObject ArrayColReaderType = {
     (getiterfunc)array_col_reader_iternext,        /*tp_iternext*/
     array_col_reader_methods,                      /*tp_methods*/
     array_col_reader_memberlist,                   /*tp_members*/
-    0,                                      /*tp_getset*/
+    0,                       /* tp_getset */
+    0,                       /* tp_base */
+    0,                       /* tp_dict */
+    0,                       /* tp_descr_get */
+    0,                       /* tp_descr_set */
+    0,                       /* tp_dictoffset */
+    0,      /* tp_init */
+    0,                         /* tp_alloc */
+    array_col_reader_new,                 /* tp_new */
 };
